@@ -129,6 +129,13 @@ Registering New Special Checks
 It is also possible for additional special check types to be registered
 using the :func:`~oslo_policy.policy.register` function.
 
+The following classes can be used as parents for custom special check types:
+
+    * :class:`~oslo_policy.policy.AndCheck`
+    * :class:`~oslo_policy.policy.NotCheck`
+    * :class:`~oslo_policy.policy.OrCheck`
+    * :class:`~oslo_policy.policy.RuleCheck`
+
 Policy Rule Expressions
 ~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -209,10 +216,10 @@ from oslo_config import cfg
 from oslo_serialization import jsonutils
 import six
 
+from oslo_policy import _cache_handler
 from oslo_policy import _checks
 from oslo_policy._i18n import _
 from oslo_policy import _parser
-from oslo_policy.openstack.common import fileutils
 from oslo_policy import opts
 
 
@@ -222,6 +229,11 @@ LOG = logging.getLogger(__name__)
 register = _checks.register
 Check = _checks.Check
 
+AndCheck = _checks.AndCheck
+NotCheck = _checks.NotCheck
+OrCheck = _checks.OrCheck
+RuleCheck = _checks.RuleCheck
+
 
 class PolicyNotAuthorized(Exception):
     """Default exception raised for policy enforcement failure."""
@@ -302,7 +314,7 @@ class Enforcer(object):
 
     :param conf: A configuration object.
     :param policy_file: Custom policy file to use, if none is
-                        specified, ``conf.policy_file`` will be
+                        specified, ``conf.oslo_policy.policy_file`` will be
                         used.
     :param rules: Default dictionary / Rules to use. It will be
                   considered just in the first instantiation. If
@@ -331,6 +343,8 @@ class Enforcer(object):
         self.use_conf = use_conf
         self.overwrite = overwrite
         self._loaded_files = []
+        self._policy_dir_mtimes = {}
+        self._file_cache = {}
 
     def set_rules(self, rules, overwrite=True, use_conf=False):
         """Create a new :class:`Rules` based on the provided dict of rules.
@@ -351,12 +365,17 @@ class Enforcer(object):
             self.rules.update(rules)
 
     def clear(self):
-        """Clears :class:`Enforcer` rules, policy's cache and policy's path."""
+        """Clears :class:`Enforcer` contents.
+
+        This will clear this instances rules, policy's cache, file cache
+        and policy's path.
+        """
         self.set_rules({})
-        fileutils.delete_cached_file(self.policy_path)
         self.default_rule = None
         self.policy_path = None
         self._loaded_files = []
+        self._policy_dir_mtimes = {}
+        self._file_cache.clear()
 
     def load_rules(self, force_reload=False):
         """Loads policy_path's rules.
@@ -380,9 +399,29 @@ class Enforcer(object):
                     path = self._get_policy_path(path)
                 except cfg.ConfigFilesNotFoundError:
                     continue
-                self._walk_through_policy_directory(path,
-                                                    self._load_policy_file,
-                                                    force_reload, False)
+                if (force_reload or self._is_directory_updated(
+                        self._policy_dir_mtimes, path)):
+                    self._walk_through_policy_directory(path,
+                                                        self._load_policy_file,
+                                                        force_reload, False)
+
+    @staticmethod
+    def _is_directory_updated(cache, path):
+        # Get the current modified time and compare it to what is in
+        # the cache and check if the new mtime is greater than what
+        # is in the cache
+        mtime = 0
+        if os.path.exists(path):
+            # Make a list of all the files
+            files = [path] + [os.path.join(path, file) for file in
+                              os.listdir(path)]
+            # Pick the newest one, let's use its time.
+            mtime = os.path.getmtime(max(files, key=os.path.getmtime))
+        cache_info = cache.setdefault(path, {})
+        if mtime > cache_info.get('mtime', 0):
+            cache_info['mtime'] = mtime
+            return True
+        return False
 
     @staticmethod
     def _walk_through_policy_directory(path, func, *args):
@@ -393,14 +432,13 @@ class Enforcer(object):
             func(os.path.join(path, policy_file), *args)
 
     def _load_policy_file(self, path, force_reload, overwrite=True):
-            reloaded, data = fileutils.read_cached_file(
-                path, force_reload=force_reload)
-            if reloaded or not self.rules or not overwrite:
-                rules = Rules.load_json(data, self.default_rule)
-                self.set_rules(rules, overwrite=overwrite, use_conf=True)
-                self._loaded_files.append(path)
-                LOG.debug('Reloaded policy file: %(path)s',
-                          {'path': path})
+        reloaded, data = _cache_handler.read_cached_file(
+            self._file_cache, path, force_reload=force_reload)
+        if reloaded or not self.rules:
+            rules = Rules.load_json(data, self.default_rule)
+            self.set_rules(rules, overwrite=overwrite, use_conf=True)
+            self._loaded_files.append(path)
+            LOG.debug('Reloaded policy file: %(path)s', {'path': path})
 
     def _get_policy_path(self, path):
         """Locate the policy JSON data file/path.
@@ -460,7 +498,7 @@ class Enforcer(object):
                 # Evaluate the rule
                 result = self.rules[rule](target, creds, self)
             except KeyError:
-                LOG.debug('Rule [%s] does not exist' % rule)
+                LOG.debug('Rule [%s] does not exist', rule)
                 # If the rule doesn't exist, fail closed
                 result = False
 
