@@ -15,14 +15,17 @@
 
 """Test of Policy Engine"""
 
+import os
+
 import mock
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslotest import base as test_base
+import six
 
+from oslo_policy import _cache_handler
 from oslo_policy import _checks
 from oslo_policy import _parser
-from oslo_policy.openstack.common import fileutils
 from oslo_policy import policy
 from oslo_policy.tests import base
 
@@ -146,9 +149,9 @@ class EnforcerTest(base.PolicyBaseTestCase):
 
     def check_loaded_files(self, filenames):
         self.assertEqual(
-            self.enforcer._loaded_files,
             [self.get_config_file_fullname(n)
-             for n in filenames]
+             for n in filenames],
+            self.enforcer._loaded_files
         )
 
     def test_load_file(self):
@@ -172,6 +175,61 @@ class EnforcerTest(base.PolicyBaseTestCase):
             'policy.d/b.conf',
         ])
 
+    def test_load_directory_caching_with_files_updated(self):
+        self.create_config_file('policy.d/a.conf', POLICY_A_CONTENTS)
+
+        self.enforcer.load_rules(False)
+        self.assertIsNotNone(self.enforcer.rules)
+
+        old = six.next(six.itervalues(
+            self.enforcer._policy_dir_mtimes))
+        self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
+
+        # Touch the file
+        conf_path = os.path.join(self.config_dir, 'policy.d/a.conf')
+        stinfo = os.stat(conf_path)
+        os.utime(conf_path, (stinfo.st_atime + 10, stinfo.st_mtime + 10))
+
+        self.enforcer.load_rules(False)
+        self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
+        self.assertEqual(old, six.next(six.itervalues(
+            self.enforcer._policy_dir_mtimes)))
+
+        loaded_rules = jsonutils.loads(str(self.enforcer.rules))
+        self.assertEqual('is_admin:True', loaded_rules['admin'])
+        self.check_loaded_files([
+            'policy.json',
+            'policy.d/a.conf',
+            'policy.d/a.conf',
+        ])
+
+    def test_load_directory_caching_with_files_same(self, overwrite=True):
+        self.enforcer = policy.Enforcer(self.conf, overwrite=overwrite)
+
+        self.create_config_file('policy.d/a.conf', POLICY_A_CONTENTS)
+
+        self.enforcer.load_rules(False)
+        self.assertIsNotNone(self.enforcer.rules)
+
+        old = six.next(six.itervalues(
+            self.enforcer._policy_dir_mtimes))
+        self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
+
+        self.enforcer.load_rules(False)
+        self.assertEqual(1, len(self.enforcer._policy_dir_mtimes))
+        self.assertEqual(old, six.next(six.itervalues(
+            self.enforcer._policy_dir_mtimes)))
+
+        loaded_rules = jsonutils.loads(str(self.enforcer.rules))
+        self.assertEqual('is_admin:True', loaded_rules['admin'])
+        self.check_loaded_files([
+            'policy.json',
+            'policy.d/a.conf',
+        ])
+
+    def test_load_directory_caching_with_files_same_but_overwrite_false(self):
+        self.test_load_directory_caching_with_files_same(overwrite=False)
+
     def test_load_multiple_directories(self):
         self.create_config_file('policy.d/a.conf', POLICY_A_CONTENTS)
         self.create_config_file('policy.d/b.conf', POLICY_B_CONTENTS)
@@ -207,16 +265,14 @@ class EnforcerTest(base.PolicyBaseTestCase):
                           self.enforcer.set_rules,
                           'dummy')
 
-    @mock.patch.object(fileutils, 'delete_cached_file', mock.Mock())
+    @mock.patch.object(_cache_handler, 'delete_cached_file', mock.Mock())
     def test_clear(self):
         # Make sure the rules are reset
         self.enforcer.rules = 'spam'
-        filename = self.enforcer.policy_path
         self.enforcer.clear()
         self.assertEqual({}, self.enforcer.rules)
         self.assertEqual(None, self.enforcer.default_rule)
         self.assertEqual(None, self.enforcer.policy_path)
-        fileutils.delete_cached_file.assert_called_once_with(filename)
 
     def test_rule_with_check(self):
         rules_json = """{
@@ -285,6 +341,7 @@ class EnforcerTest(base.PolicyBaseTestCase):
                                 use_conf=True)
 
         self.enforcer.overwrite = False
+        self.enforcer._is_directory_updated = lambda x, y: True
 
         # Call enforce(), it will load rules from
         # policy configuration files, to merge with
@@ -437,3 +494,23 @@ class RegisterCheckTestCase(base.PolicyBaseTestCase):
         policy.register('spam', TestCheck)
 
         self.assertEqual(dict(spam=TestCheck), _checks.registered_checks)
+
+
+class BaseCheckTypesTestCase(base.PolicyBaseTestCase):
+
+    @mock.patch.object(_checks, 'registered_checks', {})
+    def test_base_check_types_are_public(self):
+        '''Check that those check types are part of public API.
+
+           They are blessed to be used by library consumers.
+        '''
+        for check_type in (policy.AndCheck, policy.NotCheck,
+                           policy.OrCheck, policy.RuleCheck):
+            class TestCheck(check_type):
+                pass
+
+            check_str = str(check_type)
+            policy.register(check_str, TestCheck)
+            self.assertEqual(
+                TestCheck, _checks.registered_checks[check_str],
+                message='%s check type is not public.' % check_str)
